@@ -5,6 +5,8 @@
 #
 
 package Win32::API::Test;
+use strict;
+use warnings;
 
 sub is_perl_64bit () {
     use Config;
@@ -14,6 +16,16 @@ sub is_perl_64bit () {
     return;
 }
 
+sub can_fork () {
+    use Config;
+
+    my $native = $Config{d_fork} || $Config{d_pseudofork};
+    my $win32 = ($^O eq 'MSWin32' || $^O eq 'NetWare');
+    my $ithr = $Config{useithreads} and $Config{ccflags} =~ /-DPERL_IMPLICIT_SYS/;
+
+    return $native || ($win32 and $ithr);
+}
+
 sub compiler_name () {
     use Config;
     my $cc = $Config{ccname};
@@ -73,7 +85,7 @@ sub compiler_version_from_shell () {
 
 sub find_test_dll {
     require File::Spec;
-
+    my $dll;
     my $default_dll_name =
         is_perl_64bit()
         ? 'API_test64.dll'
